@@ -74,7 +74,8 @@ static void     pager_plugin_mode_changed                 (XfcePanelPlugin     *
 static void     pager_plugin_configure_workspace_settings (GtkWidget         *button);
 static void     pager_plugin_configure_plugin             (XfcePanelPlugin   *panel_plugin);
 static gpointer pager_plugin_get_master_plugin            (PagerPlugin       *plugin);
-static void     pager_plugin_screen_layout_changed        (PagerPlugin       *plugin);
+static void     pager_plugin_screen_layout_changed        (PagerPlugin       *plugin,
+                                                           gpointer           screen);
 static void     pager_plugin_get_preferred_width          (GtkWidget           *widget,
                                                            gint                *minimum_width,
                                                            gint                *natural_width);
@@ -293,7 +294,7 @@ pager_plugin_set_property (GObject      *object,
     case PROP_MINIATURE_VIEW:
       plugin->miniature_view = g_value_get_boolean (value);
       if (plugin->wnck_screen != NULL)
-        pager_plugin_screen_layout_changed (plugin);
+        pager_plugin_screen_layout_changed (plugin, NULL);
       break;
 
     case PROP_ROWS:
@@ -330,7 +331,7 @@ pager_plugin_set_property (GObject      *object,
               g_object_set (master_plugin, "rows", rows, NULL);
             }
           else
-            pager_plugin_screen_layout_changed (plugin);
+            pager_plugin_screen_layout_changed (plugin, NULL);
         }
       break;
 
@@ -511,12 +512,10 @@ pager_plugin_set_ratio (PagerPlugin *plugin)
       return;
     }
 
+  plugin->ratio = (gfloat) wnck_screen_get_width (plugin->wnck_screen)
+                  / (gfloat) wnck_screen_get_height (plugin->wnck_screen);
   if (wnck_workspace_is_virtual (workspace))
-    plugin->ratio = (gfloat) wnck_workspace_get_width (workspace)
-                    / (gfloat) wnck_workspace_get_height (workspace);
-  else
-    plugin->ratio = (gfloat) wnck_screen_get_width (plugin->wnck_screen)
-                    / (gfloat) wnck_screen_get_height (plugin->wnck_screen);
+    plugin->ratio *= wnck_workspace_get_width (workspace) / wnck_screen_get_width (plugin->wnck_screen);
 }
 
 
@@ -535,7 +534,7 @@ pager_plugin_screen_layout_changed_idle (gpointer data)
       return TRUE;
     }
 
-  pager_plugin_screen_layout_changed (plugin);
+  pager_plugin_screen_layout_changed (plugin, NULL);
   plugin->sync_wait = TRUE;
   plugin->sync_idle_id = 0;
 
@@ -545,7 +544,8 @@ pager_plugin_screen_layout_changed_idle (gpointer data)
 
 
 static void
-pager_plugin_screen_layout_changed (PagerPlugin *plugin)
+pager_plugin_screen_layout_changed (PagerPlugin *plugin,
+                                    gpointer screen)
 {
   XfcePanelPluginMode mode;
   GtkOrientation      orientation;
@@ -555,7 +555,8 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin)
 
   /* changing workspace layout is delayed in Libwnck code, so we have to give time
    * to the master plugin request to be processed */
-  if (plugin != pager_plugin_get_master_plugin (plugin) && plugin->sync_idle_id == 0)
+  if ((plugin != pager_plugin_get_master_plugin (plugin) || screen != NULL)
+      && plugin->sync_idle_id == 0)
     {
       plugin->sync_idle_id =
         g_idle_add_full (G_PRIORITY_LOW, pager_plugin_screen_layout_changed_idle, plugin, NULL);
@@ -634,12 +635,16 @@ pager_plugin_screen_changed (GtkWidget *widget,
     {
       plugin->wnck_screen = wnck_screen;
 
-      pager_plugin_screen_layout_changed (plugin);
+      pager_plugin_screen_layout_changed (plugin, NULL);
 
       g_signal_connect_object (G_OBJECT (screen), "monitors-changed",
          G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
       g_signal_connect_object (G_OBJECT (screen), "size-changed",
          G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
+      g_signal_connect_object (G_OBJECT (wnck_screen), "window-manager-changed",
+         G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
+      g_signal_connect_object (G_OBJECT (wnck_screen), "viewports-changed",
+         G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
     }
 }
 
@@ -768,31 +773,16 @@ pager_plugin_configure_n_workspaces_changed (WnckScreen    *wnck_screen,
                                              WnckWorkspace *workspace,
                                              GtkBuilder    *builder)
 {
-  GObject       *object;
-  gdouble        upper, value;
-  WnckWorkspace *active_ws;
+  GObject *object;
+  gdouble upper, value;
 
   panel_return_if_fail (WNCK_IS_SCREEN (wnck_screen));
   panel_return_if_fail (GTK_IS_BUILDER (builder));
 
   object = gtk_builder_get_object (builder, "rows");
-
   upper = wnck_screen_get_workspace_count (wnck_screen);
-  if (upper == 1)
-    {
-      /* check if we ware in viewport mode */
-      active_ws = wnck_screen_get_active_workspace (wnck_screen);
-      if (wnck_workspace_is_virtual (active_ws))
-        {
-          /* number of rows * number of columns */
-          upper = (wnck_workspace_get_width (active_ws) / wnck_screen_get_width (wnck_screen))
-                  * (wnck_workspace_get_height (active_ws) / wnck_screen_get_height (wnck_screen));
-        }
-    }
-
   value = MIN (gtk_adjustment_get_value (GTK_ADJUSTMENT (object)), upper);
-
-  g_object_set (G_OBJECT (object), "upper", upper, "value", value, NULL);
+  g_object_set (object, "upper", upper, "value", value, NULL);
 }
 
 
